/************************************************************************/
/*  Command line parsing routines.					*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: parse_cmdline.c,v 1.7 2000/10/26 13:39:55 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "qlib2.h"

#include "holdings.h"
#include "procs.h"
#include "externals.h"

#define	argc	(*pargc)
#define	argv	(*pargv)

/************************************************************************/
/*  parse_cmdline:							*/
/*	Parse command line.						*/
/************************************************************************/
int parse_cmdline 
   (int		*pargc,		/* ptr to argc.				*/
    char	***pargv)	/* ptr to argv.				*/
{
    char    *span;
    int	    span_flag = 0;
    char    *station_list;
    char    *creject_list = "";
    char    *sreject_list = "";
    char    *p, **pc;
    INT_TIME	*pt;
    int	    i, nc;

    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;

    cmdname = tail(*argv);

    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hd:H:D:I:R:f:t:s:")) != -1)
	switch (c) {
	case '?':
	case 'h':   print_syntax(cmdname,syntax,info); exit(0); break;
	case 'd':   debug_option = atoi(optarg); break;
	case 'H':   holdings_file = optarg; break;
	case 'D':   holdings_dir = optarg; break;
	case 'I':   creject_list = optarg; break;
	case 'R':   sreject_list = optarg; break;
	case 'f':
	    if ((pt = parse_date (optarg)) == NULL) {
		fprintf (stderr, "invalid date: %s\n", optarg);
		exit(1);
	    }
	    start_time = *pt;
	    ++start_flag;
	    break;
	case 't':
	    if ((pt = parse_date (optarg)) == NULL) {
		fprintf (stderr, "invalid date: %s\n", optarg);
		exit(1);
	    }
	    end_time = *pt;
	    ++end_flag;
	    break;
	case 's':
	    span = optarg; ++span_flag; break;
	}


    if (holdings_file == NULL && holdings_dir == NULL) {
	fprintf (stderr, "Missing holdings_file or holdings_dir\n");
	print_syntax(cmdname,syntax,stderr); 
	exit(1);
    }
    if (holdings_file != NULL && holdings_dir != NULL) {
	fprintf (stderr, "holdings_file and holdings_dir are mutually exclusive\n");
	print_syntax(cmdname,syntax,stderr);
	exit(1);
    }
    if (end_flag && span_flag) {
	fprintf (stderr, "span and endtime mutually exclusive\n");
	exit(1);
    }
    if (span_flag && !start_flag) {
	fprintf (stderr, "span not valid without start time\n");
	exit(1);
    }
    if (span_flag && ! valid_span(span)) {
	    fprintf (stderr, "invalid span specification: %s\n", span);
	    exit(1);
	}
    if (span_flag) {
	end_time = end_of_span (start_time, span);
	++end_flag;
    }

    /*	Skip over all options and their arguments.			*/
    argv = &(argv[optind]);
    argc -= optind;

    /*	There must be at least 2 additional arguments.			*/
    /*	The first arguments is the station name.			*/
    /*	All remaining arguments are channel names.			*/
    /*	There must be at least 1 channel name.				*/

    if (--argc < 1) {
	fprintf (stderr, "Missing station and/or channel argument\n");
	print_syntax(cmdname,syntax,stderr);
	exit(1);
    }
    station_list = *argv++;
    uppercase(station_list);

    /* Break comma-delimited rejection_list into list of individual	*/
    /* station names, each of which may contain wildcards.		*/
    p = sreject_list;
    n_srejects = (strlen(p)) ? 1 : 0;
    while ((p = strchr(p,',')) != NULL) {
	*p = '\0';
	++p;
	++n_srejects;
    }
    /* Create a list of pointers that point to each uppercase sreject.	*/
    p_srejects = (char **)malloc (n_srejects * sizeof (char *));
    p = sreject_list;
    for (i=0; i<n_srejects; i++) {
	p_srejects[i] = p;
	p += strlen(p) + 1;
	if (strcmp (p_srejects[i],"ALL") == 0) strcpy (p_srejects[i], "*");
    }

    /* Break comma-delimited rejection_list into list of individual	*/
    /* channel names, each of which may contain wildcards.		*/
    p = creject_list;
    n_crejects = (strlen(p)) ? 1 : 0;
    while ((p = strchr(p,',')) != NULL) {
	*p = '\0';
	++p;
	++n_crejects;
    }
    /* Create a list of pointers that point to each uppercase reject.	*/
    p_crejects = (char **)malloc (n_crejects * sizeof (char *));
    p = creject_list;
    for (i=0; i<n_crejects; i++) {
	p_crejects[i] = p;
	p += strlen(p) + 1;
	if (strcmp (p_crejects[i],"ALL") == 0) strcpy (p_crejects[i], "*");
    }

    /* Break comma-delimited station_list into list of individual	*/
    /* station names, each of which may contain wildcards.		*/
    p = station_list;
    n_stations = 1;
    while ((p = strchr(p,',')) != NULL) {
	*p = '\0';
	++p;
	++n_stations;
    }
    /* Create a list of pointers that point to each uppercase station.	*/
    p_stations = (char **)malloc (n_stations * sizeof (char *));
    p = station_list;
    for (i=0; i<n_stations; i++) {
	p_stations[i] = p;
	p += strlen(p) + 1;
	if (strcmp (p_stations[i],"ALL") == 0) strcpy (p_stations[i], "*");
    }

    /* Break all remaining channel tokens, each of which may be a	*/
    /* comma-delimited channel_listsm into list of individual		*/
    /* channel names, each of which may contain wildcards.		*/
    nc = argc;
    pc = argv;

    /* Count the number of individual channels listed.			*/
    n_channels = 0;
    for (i=0; i<nc; i++) {
	n_channels++;
	p = pc[i];
	while ((p = strchr(p,',')) != NULL) {
	    ++p;
	    n_channels++;
	}
    }

    /* Create a list of pointers that point to each uppercase station.	*/
    p_channels = (char **)malloc (n_channels * sizeof (char *));

    /* Break comma-delimited tokens into individual strings, and fill	*/
    /* in pointer list to each individual string.			*/
    n_channels= 0;
    for (i=0; i<nc; i++) {
	uppercase(pc[i]);
	p_channels[n_channels++] = pc[i];
	p = pc[i];
	while ((p = strchr(p,',')) != NULL) {
	    *p = '\0';
	    p_channels[n_channels++] = ++p;
	}
    }

    return(0);
}
